#!/bin/sh

###################################################
# Copyright 2015 VMware, Inc.  All rights reserved.
###################################################


if [ "`id -u`" != "0" ]; then
   echo "LogoutSession.sh must be executed as superuser"
   exit 1
fi

TRY_MAX=15
tryCount=0
isForce="false"


exec_logout_session() {
   ENV_XAUTHORITY="XAUTHORITY=$XAUTHORITY"
   case $desktopType in
   $MATE_DESKTOP)
      which mate-session-save >/dev/null 2>&1 && su "$UserName" -c "export '$ENV_XAUTHORITY'; mate-session-save --force-logout" >/dev/null 2>&1 && return
   ;;
   $GNOME_DESKTOP)
      which gnome-session-quit >/dev/null 2>&1 && su "$UserName" -c "export '$ENV_XAUTHORITY'; gnome-session-quit --logout --force" >/dev/null 2>&1 && return
      which gnome-session-save >/dev/null 2>&1 && su "$UserName" -c "export '$ENV_XAUTHORITY'; gnome-session-save --force-logout" >/dev/null 2>&1 && return
   ;;
   $KDE_DESKTOP)
      which qdbus >/dev/null 2>&1 && su "$UserName" -c "export '$ENV_XAUTHORITY'; qdbus org.kde.ksmserver /KSMServer org.kde.KSMServerInterface.logout 0 0 2" >/dev/null 2>&1 && return
   ;;
   *)
      echo "Logout failed: unknown desktop type $desktopType"
      return
   ;;
   esac
}

# Bug 2093253: Some times, we can't monitor the logon event successful.
# In this case, we have to restart the display manager to destroy the
# desktop session.
exec_logout_force() {
   echo "Begin to restart the display manager."
   pidof gdm >/dev/null 2>&1
   [ "$?" = "0" ] && service gdm restart

   pidof gdm-binary >/dev/null 2>&1
   [ "$?" = "0" ] && [ -f "/usr/sbin/gdm-restart" ] && /usr/sbin/gdm-restart

   pidof lightdm >/dev/null 2>&1
   [ "$?" = "0" ] && service lightdm restart
}


#
# Main
#

while getopts u:t:d:f o
do
   case $o in
   u)
      UserName="$OPTARG";;
   t)
      type="$OPTARG";;
   d)
      desktopType="$OPTARG";;
   f)
      isForce="true";;
   esac
done

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
define_constants

# Make sure the dbus monitor process is killed before logout
if [ "$type" != "broker" ]; then
   waitDBUSMonitorExit=0
   while [ $waitDBUSMonitorExit -lt 3 ]
   do
      dbusMonitorPid=`pidof -x SessionManager.py`
      if [ "$?" = "0" ]; then
         sleep 1
         waitDBUSMonitorExit=`expr $waitDBUSMonitorExit + 1`
      else
         break
      fi
   done
fi

dbusMonitorPid=`pidof -x SessionManager.py`
if [ "$?" = "0" ]; then
   kill -TERM $dbusMonitorPid
fi

# Only when the logout request from the broker, we need to run the logout command.
if [ "$type" = "broker" ]; then
   if [ "$isForce" != "true" ]; then
      exec_logout_session
   else
      exec_logout_force
      exit 0
   fi
fi

#
# We try to logout system via the force logout command.
# But some process of this session may not be killed. So
# we have to use pkill to end these remnant processes.
#

if [ -n "$VMW_SESSION_ID" ]; then
   #
   # We wait the desktop session process finished
   # at most 15 seconds
   #
   while [ $tryCount -lt $TRY_MAX ]
   do
      tryCount=`expr $tryCount + 1`
      ps -p $VMW_SESSION_ID >/dev/null 2>&1
      [ "$?" != "0" ] && break;
      sleep 1
   done

  if [ "$desktopType" = "$KDE_DESKTOP" ]; then
      #
      # Workaround for a KDE issue:
      # If there are some un-saved date in the appplication, the KDE logout command can't kill the application forcely.
      # An error dialog will be popped if kill the KDE session. The workaround is that kill the X-Server when the KDE session still exist.
      #
      pidof -s ksmserver
      if [ "$?" = "0" ]; then
         xpid="`pidof X`"
         [ "$?" = "0" ] || xpid="`pidof Xorg`"
         [ "$?" = "0" ] && kill -term $xpid
      fi
   else
      #
      # We use UserName rather than environment USERNAME, in order to prevent
      # logged in user from manipulating envInfo file. Please refer to Bug 1791847.
      #
      su "$UserName" -c "pkill -SIGHUP -s $VMW_SESSION_ID" >/dev/null 2>&1
   fi
fi
