#!/bin/bash

###################################################
# Copyright 2017 VMware, Inc.  All rights reserved.
###################################################

if [ "`id -u`" != "0" ]; then
   #
   # Requires root permission to run SsoConfigDesktopType.sh.
   #
   echo "SsoConfigDesktopType.sh must be executed as superuser"
   exit 1
fi

configure_desktop_type() {
   ret=""
   type=`awk '/^SSODesktopType/{print}' ${CUSTOM_CFG_FILE} | cut -f2 -d"=" | tail -n1`
   if [ "$?" = "0" ] && [ ! -z "$type" ]; then
      case $type in
         "UseKdePlasma")
            ret="1-kde-plasma-standard"
            ;;
         "UseGnomeClassic")
            ret="gnome-classic"
            ;;
         *)
            ;;
      esac
   fi

   uid=`id -u $username` >/dev/null 2>&1
   if [ "$?" = "0" ] && [ ! -z "$ret" ]; then
      echo "Configuration $username ($uid) with desktop $ret"
      gdbus call --system --dest org.freedesktop.Accounts --object-path /org/freedesktop/Accounts/User${uid} --method org.freedesktop.Accounts.User.SetXSession "${ret}" >/dev/null 2>&1
   fi
}

#
# Main
#
if [ "$#" != "1" ]; then
   echo "SsoConfigDesktopType.sh username"
   exit 1
fi
username="$1"

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
identify_distribution
define_constants

case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS} |\
   ${DISTRO_ID_RHEL_CLIENT} |\
   ${DISTRO_ID_RHEL_SERVER} |\
   ${DISTRO_ID_RHEL_WORKSTATION})
      if [ ${OS_MAJOR} -ge "7" ]; then
         configure_desktop_type
      fi
      ;;
   *)
      ;;
esac