#!/bin/sh

###################################################
# Copyright 2016 VMware, Inc.  All rights reserved.
###################################################

trySleep=1.5
UserName=
tsClientDir=

#
# Make sure mount point is available.
#
tryUnmount() {
   for mount_path in `mount | grep fuse.cdrserver | awk '{print $3}'`; do
      echo "umount $mount_path"
      fusermount -u -z $mount_path
   done
}

#
# Start CDR server program.
#
startCDRServer() {
   echo "Invoke the cdrserver"

   #
   # To run CDR, we need to add user to group vmwblast and fuse
   #
   local distroID=$(identify_distribution -i)
   local gps=""

   id -nG "$UserName" | grep -qw "$vmwblastGroup" >/dev/null 2>&1
   if [ "$?" != "0" ]; then
      gps="`id $UserName | cut -d "=" -f 4 | sed 's/[0-9]*(//g' | sed 's/)//g'`"
      usermod -G "$gps,vmwblast" "$UserName"
   fi

   if [ "$distroID" = "CentOS" ] || [ "$distroID" = "RedHatEnterpriseClient" ] || [ "$distroID" = "RedHatEnterpriseServer" ] || [ "$distroID" = "RedHatEnterpriseWorkstation" ]; then
      id -nG "$UserName" | grep -qw "$fuseGroup" >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         gps="`id $UserName | cut -d "=" -f 4 | sed 's/[0-9]*(//g' | sed 's/)//g'`"
         usermod -G "$gps,fuse" "$UserName"
      fi
   fi

   su $UserName -c "$CDR_SERVER &"
   ret=$?
   cdrserverPID=`pidof cdrserver`
}

#
# Main
#

if [ "`id -u`" != "0" ]; then
   #
   # Requires permission to run usermod.
   #
   echo "StartCDRServer.sh must be executed as superuser"
   exit 1
fi

while getopts u: o
do
   case $o in
   u)
      UserName="$OPTARG";;
   esac
done


echo "StartCDRServer.sh pid is: $$"

#
# Prevent from the cdr server launched duplicately
#
CDRServerID="$(pidof cdrserver)"
if [ -n "$CDRServerID" ]; then
   echo "Kill the running cdrserver: $CDRServerID"
   kill -TERM $CDRServerID
fi

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
define_constants

if [ ! -f "$CDR_SERVER" ]; then
   echo "The cdr is not supported"
   exit 0
fi

#
# Manually remove tsclient as a protection
#
tsClientDir="$(eval echo ~${UserName})/tsclient"
if [ -d "$tsClientDir" ]; then
   # Make sure the directory is empty
   if [ "$(ls -A $tsClientDir)" = "" ]; then
      rm -rf $tsClientDir
   fi
fi

while :
do
   if [ -f "/tmp/vvcProxy/vvcProxyHubInited" ]; then
      break
   else
      sleep 0.5
   fi
done

tryUnmount
startCDRServer
sleep $trySleep

echo "The cdrserver pid is: $cdrserverPID"

exit $ret
