#!/bin/sh

###################################################
# Copyright 2014 VMware, Inc.  All rights reserved.
###################################################


if [ "`id -u`" != "0" ]; then
   #
   # Requires permission to access the XAUTHORITY file.
   #
   echo "StopSession.sh must be executed as superuser"
   exit 1
fi

#
# Main
#

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
define_constants

while getopts e:p:m:t:u:f o
do
   case $o in
   e)
      env="$OPTARG";;
   p)
      pid="$OPTARG";;
   m)
      mode="$OPTARG";;
   t)
      type="$OPTARG";;
   u)
      user="$OPTARG";;
   f)
      isForce="true";;
   esac
done

# Improve the robustness. Release the clipboard resource.
startClipboardID="$(pidof -x StartClipboard.sh)"
if [ -n "$startClipboardID" ]; then
   kill -TERM $startClipboardID
fi

clipboardServerID="$(pidof mksvchanserver)"
if [ -n "$clipboardServerID" ]; then
   kill -TERM $clipboardServerID
fi

if [ -S "/tmp/vvcProxy/vvcProxySocket" ]; then
   rm -f /tmp/vvcProxy/vvcProxySocket
fi

if [ -f "/tmp/vvcProxy/vvcProxyHubInited" ]; then
   rm -f /tmp/vvcProxy/vvcProxyHubInited
fi


# Improve the robustness. Release the cdrserver resource.
startCDRServerID="$(pidof -x StartCDRServer.sh)"
if [ -n "$startCDRServerID" ]; then
   kill -TERM $startCDRServerID
fi

cdrserverID="$(pidof cdrserver)"
if [ -n "$cdrserverID" ]; then
   kill -TERM $cdrserverID
fi

#
# There are two cases that the StopSession.sh will be run:
# (1) Launch the BlasterServer failed;
# (2) Logout.
# For the case (1), we only need to kill the BlastServer.
#
if [ "$mode" != "logout" ]; then
   blastServerID="$(pidof VMwareBlastServer)"
   if [ -n "$blastServerID" ]; then
      kill -TERM $blastServerID
   fi
   exit 0
fi


if [ "$isForce" != "true" ]; then
   if [ -n "$env" ]; then
      export $env
   else
      while read -r line
      do
         # On SLED, need to ignore environment viariable which include special character like "{,},(,)"
         echo "$line" | grep -E "\{|\}|\(|\)"
         if [ "$?" = "0" ]; then
            continue
         fi
         var=`echo ${line} |cut -d= -f1`
         if check_environment_whitelisted $var; then
            export "$line"
         else
            continue
         fi
      done < "$VIEWAGENT_TMP_ROOT/view.$user/envInfo"
      #
      # The "USERNAME" is not a default environment variable on the Ubuntu 12.04.
      # As the "USERNAME" is necessary for commonds, such as: gnome-session-quit, gnome-session-save, and so on.
      # Here we need to check "USERNAME" is empty or not. If yes, we need to set the value of "USER" for it.
      #
      if [ -z "$USERNAME" ]; then
         export "USERNAME=$USER"
      fi
   fi

   desktopType="$(cat $VIEWAGENT_TMP_ROOT/view.$user/desktopType)"
fi

#
# Remove the temporary directory including files inside it
#
if [ -f "$VIEWAGENT_TMP_ROOT/view.$user/envInfo" ]; then
   rm -rf "$VIEWAGENT_TMP_ROOT/view.$user"
fi

#
# Clean the content of /var/vmware/viewagent/logEvent
#
if [ -f "$VIEWAGENT_TMP_PATH/logEvent" ]; then
   > $VIEWAGENT_TMP_PATH/logEvent
fi

#
# Clean the content of /var/vmware/viewagent/screenLockStatus
#
if [ -f "$VIEWAGENT_TMP_PATH/screenLockStatus" ]; then
   > $VIEWAGENT_TMP_PATH/screenLockStatus
fi

#
# Kill the VMwareBlastServer
#
if [ -n "$pid" ]; then
   kill $pid
else
   #
   # Wait for the blastServer exit.
   # If the blastServer doesn't exit in 10 secs, we need to kill it.
   #
   waitBlastServerExit=0
   waitBlasterServerTimeout=3
   while [ $waitBlastServerExit -lt $waitBlasterServerTimeout ]
   do
      blastServer=$(pidof VMwareBlastServer)
      if [ -z "$blastServer" ]; then
         break
      fi
      sleep 1
      waitBlastServerExit=`expr $waitBlastServerExit + 1`
   done
   if [ $waitBlastServerExit -ge $waitBlasterServerTimeout ]; then
      blastServerID="$(pidof VMwareBlastServer)"
      if [ -n "$blastServerID" ]; then
         kill -TERM $blastServerID
      fi
   fi
fi

if [ "$mode" = "logout" ]; then
   if [ "$isForce" != "true" ]; then
      /usr/lib/vmware/viewagent/bin/LogoutSession.sh -u "$user" -t $type -d $desktopType &
   else
      /usr/lib/vmware/viewagent/bin/LogoutSession.sh -u "$user" -t $type -f &
   fi
fi

# Workaround for bug 1649484.
# Sometimes mksvchanserver crash when logoff. Move away the crash files if exists.
moveVchanCrashFile(){
   crashFile=`ls /var/crash | grep _usr_lib_vmware_viewagent_VVC_vchan_plugins_mksvchanserver`
   count=0
   while [ "$crashFile" = "" -a "$count" -lt 10 ]
   do
      sleep 1s
      crashFile=`ls /var/crash | grep _usr_lib_vmware_viewagent_VVC_vchan_plugins_mksvchanserver`
      count=`expr $count + 1`
   done

   if [ "$crashFile" != "" ]; then
      mv /var/crash/_usr_lib_vmware_viewagent_VVC_vchan_plugins_mksvchanserver* /tmp/vmware-root/
      echo "crash file $crashFile"
   fi
}

if [ "$DISTRO_ID" = "$DISTRO_ID_UBUNTU" ]; then
   moveVchanCrashFile
fi

rm -f /etc/profile.d/vmware-lang.sh
originalKeyboard=`cat /var/log/vmware/keyboardLayout | awk -F ' ' '{print $1}'`
setxkbmap $originalKeyboard
rm -f /var/log/vmware/keyboardLayout
