#!/bin/bash

###################################################
# Copyright 2015 VMware, Inc.  All rights reserved.
###################################################


#
# This file should not be executed directly.
#
if [ "`basename $0`" = "commonlib.sh" ]; then
   echo "commonlib.sh is not intended to be executed directly."
   exit
fi

#
# Wrap "echo" command to dump log to either terminal/log file or both.
#
vmecho() {
   local showOnTerm=""
   local newLine=""
   local param=""
   local msgType=""
   local i=1;

   msgType="$1"

   case "$msgType" in
      term)
         showOnTerm="yes"
         ;;
      log)
         showOnTerm="no"
         ;;
   esac

   if [ "$2" = "-n" ]; then
      newLine="no"
      shift
      shift
   else
      newLine="yes"
      shift
   fi

   #
   # Collect all echo's parameters like: echo "XX" "YY" "ZZ"
   #
   while [ "$i" -le "$#" ]; do
      param="$param$1"
      i=`expr $i + 1`
   done

   #
   # Create log dir if doesn't exist
   #
   if [ ! -e "VMW_LOG_PATH" ]; then
      mkdir -p "$VMW_LOG_PATH"
      chmod 755 "$VMW_LOG_PATH"
   fi

   if [ "$showOnTerm" = "no" ]; then
      if [ "$newLine" = "yes" ]; then
         echo "$param" >> "$VIEWAGENT_INSTALLER_LOG"
      else
         echo -n "$param" >> "$VIEWAGENT_INSTALLER_LOG"
      fi
   else
      if [ "$newLine" = "yes" ]; then
         echo "$param" | tee -a "$VIEWAGENT_INSTALLER_LOG"
      else
         echo -n "$param" | tee -a "$VIEWAGENT_INSTALLER_LOG"
      fi
   fi
}

#
# Dump message to both terminal and log file.
#
echo_term() {
   vmecho term "$@"
   return 0
}

#
# Dump message to log file only.
#
echo_log() {
   vmecho log "$@"
   return 0
}

#
# Command funcions
#

identify_distribution() {
   DISTRO_ID_CENTOS=CentOS
   DISTRO_ID_RHEL_CLIENT=RedHatEnterpriseClient
   DISTRO_ID_RHEL_SERVER=RedHatEnterpriseServer
   DISTRO_ID_RHEL_WORKSTATION=RedHatEnterpriseWorkstation
   DISTRO_ID_NEOKYLIN=" NeoKylin Linux Desktop"
   DISTRO_ID_UBUNTU=Ubuntu
   DISTRO_ID_SUSE="SUSE LINUX"
   #
   # Virtual distro IDs
   #
   DISTRO_ID_KYLIN=UbuntuKylin

   type lsb_release >/dev/null 2>&1
   if [ "$?" = "0" ]; then
      DISTRO_ID="`lsb_release -is`"
      DISTRO_RELEASE="`lsb_release -rs`"
      DISTRO_DESC="`lsb_release -ds`"
   else
      if [ -f /etc/centos-release ]; then
         DISTRO_ID=$DISTRO_ID_CENTOS
         DISTRO_DESC="`cat /etc/centos-release`"
      elif [ -f /etc/redhat-release ]; then
         DISTRO_DESC="`cat /etc/redhat-release`"
         DISTRO_ID=
         echo $DISTRO_DESC | grep "Client" >/dev/null 2>&1 && \
            DISTRO_ID=$DISTRO_ID_RHEL_CLIENT
         [ -z "$DISTRO_ID" ] && \
            echo $DISTRO_DESC | grep "Server" >/dev/null 2>&1 && \
            DISTRO_ID=$DISTRO_ID_RHEL_SERVER
         [ -z "$DISTRO_ID" ] && \
            echo $DISTRO_DESC | grep "Workstation" >/dev/null 2>&1 && \
            DISTRO_ID=$DISTRO_ID_RHEL_WORKSTATION
      else
         echo "Unable to identify distribution and release."
         exit
      fi
      DISTRO_RELEASE="`echo $DISTRO_DESC | awk '{ for (i=1; i < NF; i++) { if ($i == "release") { print $(i+1); break }}}'`"
   fi

   #
   # Differentiate between Ubuntu and Ubuntu Kylin.
   #
   if [ "$DISTRO_ID" = "$DISTRO_ID_UBUNTU" ]; then
      #
      # Distinguish between Ubuntu and Ubuntu Kylin where possible.   Release
      # files are common place and the following file exists on
      # Ubuntu Kylin 14.04 LTS, but not on Ubuntu Kylin 13.04.   Your mileage
      # may vary...
      if [ -f /etc/ubuntukylin-release ]; then
         DISTRO_ID=$DISTRO_ID_KYLIN
      fi
   fi

   if [ "$DISTRO_ID" = "SUSE" ]; then
      #
      # Add the distribution ID "SUSE" to the DISTRO_ID_SUSE
      #
      DISTRO_ID=$DISTRO_ID_SUSE
   fi

   #
   # Check supported distributions
   #
   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_SUSE}|\
   ${DISTRO_ID_RHEL_WORKSTATION}|\
   ${DISTRO_ID_NEOKYLIN})
      ;;
   ${DISTRO_ID_KYLIN}|${DISTRO_ID_UBUNTU})
      echo $DISTRO_DESC | grep " LTS$" >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo "$DISTRO_DESC not an LTS release"
         exit
      fi
      ;;
   *)
      echo "$DISTRO_ID not supported"
      #exit
      ;;
   esac

   case "$1" in
      -d)
         echo "$DISTRO_DESC";;
      -r)
         echo "$DISTRO_RELEASE";;
      -i)
         echo "$DISTRO_ID";;
      *)
         ;;
   esac
}


define_constants() {
   OS_VERSION=100
   OS_MAJOR=`echo "$DISTRO_RELEASE" | cut -f1 -d"."`
   OS_MINOR=`echo "$DISTRO_RELEASE" | cut -f2 -d"."`
   VIEWAGENT_TAR="VMware-viewagent-standalone-*"
   VMWARE_PATH=/usr/lib/vmware
   VIEWAGENT_PATH=/usr/lib/vmware/viewagent
   VIEWAGENT_TMP_ROOT=/var/vmware
   VIEWAGENT_TMP_PATH=$VIEWAGENT_TMP_ROOT/viewagent
   JRE_PATH=/usr/lib/vmware/viewagent/jre
   VIEWAGENT_ENV_FILE=/etc/sysconfig/viewagent
   VIEWAGENT_ENV_FILE_UBUNTU=/etc/default/viewagent
   VMW_SSL_PATH="/etc/vmware/ssl"
   AGENT_CFG_FILE=/etc/vmware/viewagent-config.txt
   JMS_KEYSTORE_DIR=/etc/vmware/jms
   VIEWAGENT_UPSTART=viewagent.conf
   UPSTART_CONF_PATH=/etc/init
   VIEWAGENT_SYSV=viewagent
   VIEWAGENT_SYSV_SUSE=viewagent.suse
   SYSV_CONF_PATH=/etc/rc.d/init.d
   SYSV_CONF_PATH_SUSE=/etc/init.d
   VIEWAGENT_SYSTEMD=viewagent.service
   VIEWAGENT_SYSTEMD_UBUNTU=viewagent_ubuntu_systemd.service
   SYSTEMD_CONF_PATH=/usr/lib/systemd/system
   SYSTEMD_CONF_PATH_UBUNTU=/lib/systemd/system
   # shutdown service constants
   SHUTDOWN_SYSV=viewagent_shutdown
   SHUTDOWN_SYSV_SUSE=viewagent_shutdown.suse
   SHUTDOWN_SYSTEMD_UBUNTU=viewagent_shutdown_ubuntu_systemd.service
   SHUTDOWN_SYSTEMD=viewagent_shutdown.service
   SHUTDOWN_ENV_FILE=/etc/sysconfig/viewagent_shutdown
   SHUTDOWN_ENV_FILE_UBUNTU=/etc/default/viewagent_shutdown
   SHUTDOWN_UPSTART=viewagent_shutdown.conf

   VIEWAGENT_POLICYKIT=10-com.vmware.viewagent.pkla
   POLICYKIT_PATH=/etc/polkit-1/localauthority/90-mandatory.d
   PAM_PATH=/etc/pam.d
   LIGHTDM_PATH=/etc/lightdm
   GLIB_SCHEMAS_PATH=/usr/share/glib-2.0/schemas
   VMWARE_SCHEMA_OVERRIDE=50_vmware_viewagent.gschema.override
   BORA_CFG_FILE=/etc/vmware/config
   MACHINE_CFG_FILE=/etc/vmware/viewagent-machine.cfg
   START_SH=/usr/lib/vmware/viewagent/bin/StartSession.sh
   END_SH=/usr/lib/vmware/viewagent/bin/StopSession.sh
   MSG_SH=/usr/lib/vmware/viewagent/bin/DisplayMessage.sh
   START_CLIPBOARD_SH=/usr/lib/vmware/viewagent/bin/StartClipboard.sh
   STOP_CLIPBOARD_SH=/usr/lib/vmware/viewagent/bin/StopClipboard.sh
   START_CDR_SERVER_SH=/usr/lib/vmware/viewagent/bin/StartCDRServer.sh
   STOP_CDR_SERVER_SH=/usr/lib/vmware/viewagent/bin/StopCDRServer.sh
   START_USBREDIR_SH=/usr/lib/vmware/viewagent/bin/StartUsbRedir.sh
   STOP_USBREDIR_SH=/usr/lib/vmware/viewagent/bin/StopUsbRedir.sh
   CONTROL_DESKTOP_SH=/usr/lib/vmware/viewagent/bin/ControlDesktop.sh
   CLEANLOG_SH=/usr/lib/vmware/viewagent/bin/CleanupLogFiles.sh
   MOUNT_FOLDER_SH=/usr/lib/vmware/viewagent/bin/MountFolder.sh
   UMOUNT_FOLDER_SH=/usr/lib/vmware/viewagent/bin/UMountFolder.sh
   VMW_LOG_PATH=/var/log/vmware
   VIEWAGENT_REG_LOG=${VMW_LOG_PATH}/viewagent-registration.log
   VIEWAGENT_INSTALLER_LOG=${VMW_LOG_PATH}/viewagent-installer.log
   PRIVATE_KEY=$VMW_SSL_PATH/rui.key
   PRIVATE_KEY_SIZE=2048
   CERT_REQ=$VMW_SSL_PATH/rui.csr
   CERT=$VMW_SSL_PATH/rui.crt
   CERT_VALIDITY_PERIOD=1460
   BLAST_RUNTIME_DIR=/tmp/vmwblast
   BLAST_ACCOUNT=vmwblast
   BLAST_GROUP=vmwblast
   BLAST_PORT=22443
   DNPREFIX="/C=US/ST=California/O=VMware"
   VIEWAGENT_MONITOR_SCRIPT="viewagent-monitor.desktop"
   VIEWAGENT_GDM_GREETER_LOGIN_WINDOW_FILE="gdm-greeter-login-window.glade"
   VIEWAGENT_GREETER_STATUS_PATH=$VIEWAGENT_TMP_ROOT/greeter
   VIEWAGENT_CUSTOMISE_DESKTOP_LOG=/var/log/vmware/viewagent-customise.log
   VIEWAGENT_RUNTIME_DIR=/var/run/vmware/viewagent
   XORG_CONF_DIR="/etc/X11"
   VVC_PATH="$VIEWAGENT_PATH/VVC"
   VCHAN_PLUGINS_PATH="$VVC_PATH/vchan_plugins"
   HELPDESK_PATH="$VIEWAGENT_PATH/helpdesk"
   USBREDIR_SERVER="$VCHAN_PLUGINS_PATH/usbRedirectionServer"
   CLIPBOARD_SERVER="$VCHAN_PLUGINS_PATH/mksvchanserver"
   CDR_SERVER="$VCHAN_PLUGINS_PATH/cdrserver"
   SCREDIR_BACKUP_DIR="${VIEWAGENT_PATH}/scredir/backup"
   SCREDIR_IPC_DIR="/var/run/vvcipc"
   # TODO: Use blast instead of pcoip
   BLAST_LIB_PATH="/usr/lib/pcoip"
   BLAST_VCHAN_PLUGINS_PATH="$BLAST_LIB_PATH/vchan_plugins"
   SCREDIR_VCHAN_PLUGINS_PATH="$VMWARE_PATH/vchan_plugins"
   AUDIOIN_VCHAN_PLUGINS_PATH="$VMWARE_PATH/vchan_plugins"
   DBUS_CONF_PATH=/etc/dbus-1/system.d
   SSO_DBUS_PY=/usr/lib/vmware/viewagent/bin/SsoDBusNotify.py
   SSO_CONFIG_DESKTOP_TYPE_SH=/usr/lib/vmware/viewagent/bin/SsoConfigDesktopType.sh
   SSO_BACKUP_PATH="${VIEWAGENT_PATH}/sso/backup"
   # Custom configuration file of viewagent
   CUSTOM_CFG_FILE=/etc/vmware/viewagent-custom.conf
   # Custom configuration file of VMwareBlastServer
   CUSTOM_CFG_FILE_BLAST=/etc/vmware/config
   MINVERSION_XORG_X11_SERVER=7.4
   MINRELEASE_XORG_X11_SERVER=27.111.1
   MINVERSION_XF86_VIDEO_VMWARE=13.0.2
   MINRELEASE_XF86_VIDEO_VMWARE=3.2
   MINVERSION_MESA_LIBXATRACKER=10.6.5
   MINRELEASE_MESA_LIBXATRACKER=3
   MINVERSION_XORG_X11_DRV_VMWARE=13.0.2
   MINRELEASE_XORG_X11_DRV_VMWARE=7
   MINVERSION_MESA_PRIVATE_LLVM=3.6.2
   MINRELEASE_MESA_PRIVATE_LLVM=2
   MINVERSION_MESA_DRI_DRIVERS=10.6.5
   MINRELEASE_MESA_DRI_DRIVERS=3
   BROKER_MESSAGEBOX_TITLE="VMware Horizon View Administrator"
   GNOME_DESKTOP="gnome"
   MATE_DESKTOP="mate"
   KDE_DESKTOP="kde"
   NGVC_SHUTDOWN=vmwareNgvcShutDown.sh
}


define_commands() {
   which addgroup >/dev/null 2>&1
   [ "$?" = "0" ] && [ ! -n "$ADDGROUP" ] && ADDGROUP=addgroup
   which groupadd >/dev/null 2>&1
   [ "$?" = "0" ] && [ ! -n "$ADDGROUP" ] && ADDGROUP=groupadd
}


configure_distribution_common() {
   #
   # Generic
   #
   define_constants

   #
   # Java binary
   #
   JRE_JAVA="$JRE_PATH/bin/java"

   #
   # Distro specific UI
   #
   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS})
      DISTRO_PLATFORM_LABEL=CentOS
      ;;
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})
      DISTRO_PLATFORM_LABEL=RHEL
      ;;
   ${DISTRO_ID_NEOKYLIN})
      DISTRO_PLATFORM_LABEL=FedoraKylin
      ;;
   ${DISTRO_ID_UBUNTU})
      DISTRO_PLATFORM_LABEL=Ubuntu
      ;;
   ${DISTRO_ID_KYLIN})
      DISTRO_PLATFORM_LABEL=UbuntuKylin
      ;;
   ${DISTRO_ID_SUSE})
      DISTRO_PLATFORM_LABEL=SUSE
      ;;
   esac

   #
   # Identify distro specific service definition and control
   #
   SERVICE_MGR=
   SERVICE_MGR_SYSV=SYSV
   SERVICE_MGR_SYSTEMD=SYSTEMD
   SERVICE_MGR_UPSTART=UPSTART

   [ -z "$SERVICE_MGR" ] && \
      which initctl >/dev/null 2>&1 && SERVICE_MGR=${SERVICE_MGR_UPSTART}

   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})
      #
      # CentOS / RHEL prior to the introduction of systemd (< 7.0) contain
      # Upstart from Fedora however the services are still managed via SysV
      # scripts.   Ignore the availability of Upstart on such systems.
      #
      expr $DISTRO_RELEASE \< 7.0 >/dev/null
      if [ "$?" = "0" ]; then
         SERVICE_MGR=
      fi
      ;;
   ${DISTRO_ID_UBUNTU})
      if [ "$OS_MAJOR" = "16" ]; then
         SERVICE_MGR=
      fi
      ;;
   esac

   [ -z "$SERVICE_MGR" ] && \
      which systemctl >/dev/null 2>&1 && SERVICE_MGR=${SERVICE_MGR_SYSTEMD}

   [ -z "$SERVICE_MGR" ] && SERVICE_MGR=${SERVICE_MGR_SYSV}

   #
   # Assume all distributions and releases have compatible service / chkconfig
   # commands regardless of service management employed.   Override in distro
   # specific commands section below if this is not the case.
   #
   SERVICE_START="service"
   SERVICE_START_POST="start"
   SERVICE_STOP="service"
   SERVICE_STOP_POST="stop"
   SERVICE_RESTART="service"
   SERVICE_RESTART_POST="restart"
   SERVICE_SAVE="service"
   SERVICE_SAVE_POST="save"

   #
   # Identify firewall
   #
   FIREWALL=
   FIREWALL_IPTABLES=IPTABLES
   FIREWALL_FIREWALLD=FIREWALLD

   [ -z "$FIREWALL" ] && \
      which firewall-cmd >/dev/null 2>&1 && firewall-cmd --list-all >/dev/null 2>&1 && FIREWALL=${FIREWALL_FIREWALLD}

   [ -z "$FIREWALL" ] && \
      which iptables >/dev/null 2>&1 && FIREWALL=${FIREWALL_IPTABLES}

   #
   # Initialise desktop customisation
   #

   REQUEST_DESKTOP_RESTART=

   GCONFTOOL=
   which gconftool-2 >/dev/null 2>&1 && GCONFTOOL=yes

   GLIB_SCHEMAS=
   which gsettings >/dev/null 2>&1 && [ -d "${GLIB_SCHEMAS_PATH}" ] && \
      which glib-compile-schemas >/dev/null 2>&1 && \
      GLIB_SCHEMAS=yes
   #
   # customise_desktop_gsettings does not support the earlier syntax of
   # gsettings.   Verify the verify present on the system and clear variable
   # if not supported.
   #
   [ "$GLIB_SCHEMAS" = "yes" ] && gsettings list-schemas >/dev/null 2>&1 || \
      GLIB_SCHEMAS=

   POLICYKIT=
   which pkaction >/dev/null 2>&1 && [ -d ${POLICYKIT_PATH} ] && \
      POLICYKIT=yes

   #
   # Distro specific commands
   #
   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})
      PKG_QUERY="rpm -q"
      PKG_INSTALL="yum -q -y install"
      ;;
   ${DISTRO_ID_NEOKYLIN})
      PKG_QUERY="rpm -q"
      PKG_INSTALL="yum -q -y install"
      ;;
   ${DISTRO_ID_KYLIN}|\
   ${DISTRO_ID_UBUNTU})
      PKG_QUERY="dpkg -s"
      PKG_INSTALL="apt-get -q -y install"
      ;;
   ${DISTRO_ID_SUSE})
      PKG_QUERY="rpm -q"
      PKG_INSTALL="zypper -q -n install"
      ;;
   esac
}

cleanSSOFiles() {
   local arch="`uname -i`"
   local archpath="lib64"
   local majorVersion=

   # Don't support 32 bit linux system
   [ "$arch" != "x86_64" ] && return

   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})
      majorVersion=`identify_distribution -r | head -c 1`
      ;;
   *)
      return
   esac

   if [ "$majorVersion" = "6" ]; then
      rm -f /usr/$archpath/gdm/simple-greeter/plugins/vmwcred.so

      rm -fr /usr/share/gdm/simple-greeter/extensions/vmwcred

      rm -f /usr/share/icons/hicolor/16x16/apps/gdm-vmwcred.png

      rm -f /usr/share/icons/hicolor/48x48/apps/gdm-vmwcred.png

      rm -f /$archpath/security/pam_vmw_cred.so

      rm -f /etc/pam.d/gdm-vmwcred

      # Clean the dbus conf file
      rm -f $DBUS_CONF_PATH/org.vmware.viewagent.Credentials.conf
   fi
}

registerRollback() {
   rm -f /etc/xdg/autostart/$VIEWAGENT_MONITOR_SCRIPT
   rm -fr $VMWARE_PATH

   # Clean the tmp file
   rm -rf $VIEWAGENT_TMP_PATH

   # Clean the configure files
   rm -fr /etc/vmware

   # Clean the blast lib path
   rm -fr $BLAST_LIB_PATH

   # Clean the SSO files
   cleanSSOFiles
}

serviceStart() {
   if [ "$#" != "1" ]; then
      echo "serviceStart: service not specified"
   else
      $SERVICE_START $1 $SERVICE_START_POST > /dev/null 2>&1
   fi
}


serviceRestart() {
   if [ "$#" != "1" ]; then
      echo "serviceRestart: service not specified"
   else
      $SERVICE_RESTART $1 $SERVICE_RESTART_POST > /dev/null 2>&1
   fi
}


serviceStop() {
   if [ "$#" != "1" ]; then
      echo "serviceStop: service not specified"
   else
      $SERVICE_STOP $1 $SERVICE_STOP_POST > /dev/null 2>&1
   fi
}


serviceSave() {
   if [ "$#" != "1" ]; then
      echo "serviceSave: service not specified"
   else
      $SERVICE_SAVE $1 $SERVICE_SAVE_POST > /dev/null 2>&1
   fi
}

setLogFile() {
   exec 3>&1
   exec 4>&2

   exec 1>$1
   exec 2>$1
}

resetLogFile() {
   exec 1>&3
   exec 2>&4
}

customise_desktop() {
   setLogFile $VIEWAGENT_CUSTOMISE_DESKTOP_LOG
   local mode
   local managed

   if [ "$#" != "2" ]; then
      echo "customise_desktop: specify operation"
      return
   fi

   mode="$1"
   managed="$2"

   case "$mode" in
   "set"|"reset")
      customise_desktop_gsettings $mode $managed
      customise_desktop_gconf $mode $managed
      customise_desktop_polkit $mode $managed
      customise_desktop_pam $mode $managed
      if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ] && [ "$OS_MAJOR" = "11" ]; then
         customise_desktop_SLED $mode $managed
      fi
      if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ] && [ "$OS_MAJOR" = "12" ]; then
         echo $DISTRO_DESC | grep "Server" >/dev/null 2>&1
         if [ "$?" = "0" ]; then
            customise_desktop_SLES $mode
         fi
      fi

      #
      # customise ubuntu1404 mate, disable the switch user dbus message for ubuntu 1404.
      #
      if [ "$DISTRO_ID" = "${DISTRO_ID_UBUNTU}" ] && [ "${OS_MAJOR}" = "14" ]; then
         customise_desktop_ubuntu1404_mate $mode
      fi

      #
      # Check desktop environment before install agent
      #
      ls /usr/share/xsessions | grep "kde" >/dev/null 2>&1 || \
      ls /usr/share/xsessions | grep "plasma" >/dev/null 2>&1
      if [ "$?" = "0" ]; then
         customise_desktop_kde $mode
      fi
      ;;
   *)
      echo "customise_desktop: specify set or reset"
      ;;
   esac

   if [ "$DISTRO_ID" = "$DISTRO_ID_NEOKYLIN" ]; then
      customise_desktop_NeoKylin $managed
   fi
   resetLogFile
}

customise_desktop_ubuntu1404_mate() {
   local mode
   local DBUS_CONF_FILE_PATH="/etc/dbus-1/system.d/org.freedesktop.DisplayManager.conf"
   local DBUS_CONF_FILE_PATH_ORG="/etc/dbus-1/system.d/org.freedesktop.DisplayManager.conf.org"

   if [ "$#" != "1" ]; then
      echo "customise_desktop_ubuntu1404_mate: specify operation"
      return
   fi


 mode="$1"

   case "$mode" in
   "set")
      # Disable Switch User in the logout screen
      if [ -f "$DBUS_CONF_FILE_PATH" ]; then
         # Backup system original file
         mv -f $DBUS_CONF_FILE_PATH $DBUS_CONF_FILE_PATH_ORG

         # Deploy new file
         cp -fp $VIEWAGENT_PATH/resources/mate/org.freedesktop.DisplayManager.conf $DBUS_CONF_FILE_PATH
      fi
      ;;
   "reset")
      # Restore the system original file
      if [ -f "$DBUS_CONF_FILE_PATH_ORG" ]; then
         mv -f $DBUS_CONF_FILE_PATH_ORG $DBUS_CONF_FILE_PATH
      fi
      ;;
   *)
      echo "customise_desktop_ubuntu1404_mate: specify set or reset"
      ;;
   esac
}

customise_desktop_gsettings() {
   local mode
   local schema
   local key
   local value
   local last_schema=""
   local customised=""
   local managed

   if [ "$#" != "2" ]; then
      echo "customise_desktop_gsettings: specify operation"
      return
   fi

   mode="$1"
   managed="$2"

   [ "$GLIB_SCHEMAS" = "yes" ] || return

   #
   # For reset remove viewagent gschema override file.
   #
   if [ "$mode" = "reset" ]; then
      rm -f ${GLIB_SCHEMAS_PATH}/${VMWARE_SCHEMA_OVERRIDE}
      return
   fi

   while read cfg
   do
      schema=`echo $cfg | cut -f1 -d" "`

      gsettings list-schemas | grep "$schema" >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo "customise_desktop_gsettings: Info: $schema not present"
         continue;
      fi

      key=`echo $cfg | cut -f2 -d" "`

      gsettings get $schema $key >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo "customise_desktop_gsettings: Info: $schema $key not present"
         continue;
      fi

      case "$mode" in
      "set")
         if [ "$managed" = "true" ] &&
            [ "$key" = "suppress-restart-menuitem" -o "$key" = "suppress-shutdown-menuitem" ]; then
            echo "customise_desktop_gsettings: managed agent don't supress shut down and restart menu item"
            continue
         else
            value=`echo $cfg | cut -f3 -d" "`
         fi

         if [ "$last_schema" != "$schema" ]; then
            [ -n "$customised" ] && echo >> ${GLIB_SCHEMAS_PATH}/${VMWARE_SCHEMA_OVERRIDE}
            echo "[${schema}]" >> ${GLIB_SCHEMAS_PATH}/${VMWARE_SCHEMA_OVERRIDE}
            last_schema=$schema
            customised=yes
         fi
         echo "${key}=${value}" >> ${GLIB_SCHEMAS_PATH}/${VMWARE_SCHEMA_OVERRIDE}
         ;;
      esac
   done << !!!
com.canonical.indicator.session suppress-restart-menuitem true
com.canonical.indicator.session suppress-shutdown-menuitem true
org.gnome.desktop.lockdown disable-user-switching true
org.gnome.desktop.screensaver user-switch-enabled false
org.mate.lockdown disable-user-switching true
org.mate.screensaver user-switch-enabled false
org.gnome.settings-daemon.plugins.media-keys logout '<Control><Alt>Insert'
!!!

   if [ -n "$customised" ]; then
      glib-compile-schemas ${GLIB_SCHEMAS_PATH}/
      REQUEST_DESKTOP_RESTART=yes
   else
      echo "customise_desktop_gsettings: Info: no changes"
   fi
}


customise_desktop_gconf() {
   local mode
   local schema
   local key
   local type
   local value
   local customised=""
   local file
   local DBUS_SESSION_BUS_ADDRESS_ENV
   local managed

   if [ "$#" != "2" ]; then
      echo "customise_desktop_gconf: specify operation"
      return
   fi

   mode="$1"
   managed="$2"

   [ "$GCONFTOOL" = "yes" ] || return

   #
   # Here we need a wordaround to handle "GConf Error: Failed to contact configuration server;
   # some possible causes are that you need to enable TCP/IP networking for ORBit,or you have
   # stale NFS locks due to a system crash. See http://project.gnome.org/gconf/ for information.
   # (Details -1: Failed to get connection to session: Did you receive a reply. Possible causes
   # include: the remote application did not send a reply, the message bus security policy blocked
   # the reply, the reply timeout expired, or the network connection was broken.)" on SLED
   #
   if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ]; then
      DBUS_SESSION_BUS_ADDRESS_ENV="$DBUS_SESSION_BUS_ADDRESS"
      unset DBUS_SESSION_BUS_ADDRESS
   fi

   while read cfg
   do
      schema=`echo $cfg | cut -f1 -d" "`

      gconftool-2 --dir-exists="$schema" >/dev/null 2>&1
      if [ "$?" != "0" ]; then
         echo "customise_desktop_gconf: Info: $schema not present"
         continue;
      fi

      key=`echo $cfg | cut -f2 -d" "`
      file=`echo $cfg |cut -f5 -d" "`
      if [ ! -d "/etc/gconf/${file}" ]; then
         echo "folder /etc/gconf/${file} don't exist"
         continue;
      fi

      gconftool-2 --get "/schemas${schema}/${key}" 2>&1 | \
         grep "^No value set for" >/dev/null 2>&1
      if [ "$?" = "0" ]; then
         echo "customise_desktop_gconf: Info: $schema/$key not present"
         continue;
      fi

      if [ "$managed" = "true" ] &&
         [ "$key" = "suppress-restart-menuitem" -o "$key" = "suppress-shutdown-menuitem" ]; then
         echo "managed agent don't supress restart and shut down menu item"
         continue;
      fi

      case "$mode" in
      "set")
         type=`echo $cfg | cut -f3 -d" "`
         value=`echo $cfg | cut -f4 -d" "`

         gconftool-2 --direct --config-source xml:readwrite:/etc/gconf/${file} --type $type --set ${schema}/${key} $value >/dev/null 2>&1
         [ "$?" = "0" ] && customised=yes
         ;;
      "reset")
         gconftool-2 --direct --config-source xml:readwrite:/etc/gconf/${file} --unset ${schema}/${key} >/dev/null 2>&1
         [ "$?" = "0" ] && customised=yes
         ;;
      esac
   done << !!!
/apps/gnome-screensaver user_switch_enabled bool false gconf.xml.mandatory
/apps/gnome-power-manager/general can_hibernate bool false gconf.xml.mandatory
/apps/gnome_settings_daemon/keybindings power string <Control><Alt>Insert gconf.xml.defaults
/apps/indicator-session suppress_shutdown_menuitem bool true gconf.xml.mandatory
/apps/indicator-session suppress_restart_menuitem bool true gconf.xml.mandatory
/desktop/gnome/lockdown disable_user_switching bool true gconf.xml.mandatory
!!!

   #
   # Shutting down gconfd should trigger a restart / reload
   #
   if [ -n "$customised" ]; then
      gconftool-2 --shutdown
      REQUEST_DESKTOP_RESTART=yes
   else
      echo "customise_desktop_gconf: Info: no changes"
   fi

   if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ]; then
      export "DBUS_SESSION_BUS_ADDRESS=$DBUS_SESSION_BUS_ADDRESS_ENV"
   fi
}


customise_desktop_polkit() {
   local mode
   local policy
   local description
   local action
   local last_description=""
   local last_identity=""
   local customised=""
   local pkfile=${POLICYKIT_PATH}/${VIEWAGENT_POLICYKIT}
   local managed

   if [ "$#" != "2" ]; then
      echo "customise_desktop_polkit: specify operation"
      return
   fi

   mode="$1"
   managed="$2"

   [ "$POLICYKIT" = "yes" ] || return

   #
   # For reset remove viewagent mandatory policykit file.
   #
   if [ "$mode" = "reset" ]; then
      rm -f ${pkfile}
      return
   fi

   while read policy
   do
      case "$mode" in
      "set")
         description=`echo $policy | cut -f1 -d";"`
         action=`echo $policy | cut -f2 -d";"`
         identity=`echo $policy | cut -f3 -d";"`

         if [ "$managed" = "true" ] &&
            [ "$description" = "Disable shutdown" -o "$description" = "Disable restart" ]; then
            echo "customise_desktop_polkit: managed agent don't supress shut down and restart menu item"
            continue
         fi

         pkaction 2>&1 | grep ${action} >/dev/null 2>&1
         if [ "$?" != "0" ]; then
            echo "customise_desktop_polkit: Info: Action $action not registered"
            continue
         fi

         if [ -z "$customised" ]; then
            > ${pkfile}
            chown root:root ${pkfile}
            chmod 644 ${pkfile}
         fi

         if [ "$last_description" != "$description" ] || [ "$last_identity" != "$identity" ]; then
            if [ -n "$customised" ]; then
               echo >> ${pkfile}
               echo "ResultAny=no" >> ${pkfile}
               echo "ResultInactive=no" >> ${pkfile}
               echo "ResultActive=no" >> ${pkfile}
               echo >> ${pkfile}
            fi
            echo "[${description}]" >> ${pkfile}
            echo "Identity=${identity}:*" >> ${pkfile}
            echo -n "Action=${action}" >> ${pkfile}
            last_description=$description
            last_identity=$identity
            customised=yes
         else
            echo -n ";${action}" >> ${pkfile}
         fi

         ;;
      esac
   done << !!!
Disable suspend;org.freedesktop.upower.suspend;unix-user
Disable suspend;org.freedesktop.login1.suspend;unix-user
Disable suspend;org.freedesktop.login1.suspend-multiple-sessions;unix-user
Disable suspend;org.freedesktop.devicekit.power.suspend;unix-group
Disable suspend;org.freedesktop.upower.hibernate;unix-user
Disable suspend;org.freedesktop.login1.hibernate;unix-user
Disable suspend;org.freedesktop.login1.hibernate-multiple-sessions;unix-user
Disable suspend;org.freedesktop.devicekit.power.hibernate;unix-group
Disable shutdown;org.freedesktop.consolekit.system.stop;unix-group
Disable shutdown;org.freedesktop.consolekit.system.stop-multiple-users;unix-group
Disable shutdown;org.freedesktop.login1.power-off;unix-group
Disable shutdown;org.freedesktop.login1.power-off-multiple-sessions;unix-group
Disable restart;org.freedesktop.consolekit.system.restart;unix-group
Disable restart;org.freedesktop.consolekit.system.restart-multiple-users;unix-group
Disable restart;org.freedesktop.login1.reboot;unix-group
Disable restart;org.freedesktop.login1.reboot-multiple-sessions;unix-group
!!!

   if [ -n "$customised" ]; then
      echo >> ${pkfile}
      echo "ResultAny=no" >> ${pkfile}
      echo "ResultInactive=no" >> ${pkfile}
      echo "ResultActive=no" >> ${pkfile}
      REQUEST_DESKTOP_RESTART=yes
   else
      echo "customise_desktop_polkit: Info: no changes"
   fi
}


customise_desktop_pam() {
   local mode
   local managed

   if [ "$#" != "2" ]; then
      echo "customise_desktop_pam: specify operation"
      return
   fi

   mode="$1"
   managed="$2"

   [ -d ${PAM_PATH} ] || return

   while read file
   do
      case "$mode" in
      "set")
         if [ "$managed" = "true" ] &&
            [ "$file" != "halt" ]; then
            echo "customise_desktop_pam: don't suppress shutdown restart poweroff item"
            continue;
         fi

         if [ -f ${PAM_PATH}/$file ]; then
            cp -p ${PAM_PATH}/$file ${PAM_PATH}/$file.vmw
            if [ "$?" != "0" ]; then
               echo "Failed to move ${PAM_PATH}/$file"
            else
               sed -i '1i\#vmware linux desktop disable '$file'\nauth   include  system-auth' ${PAM_PATH}/$file
            fi
         fi
         ;;
      "reset")
         if [ -f ${PAM_PATH}/$file.vmw ]; then
            mv ${PAM_PATH}/$file.vmw ${PAM_PATH}/$file
            if [ "$?" != "0" ]; then
               echo "Failed to restore ${PAM_PATH}/$file"
            fi
         fi
         ;;
      esac
   done << !!!
halt
poweroff
shutdown
reboot
!!!
}

customise_desktop_kde() {
   local mode
   local LOCK_SCREEN_FILE_PATH="/usr/share/kde4/apps/ksmserver/screenlocker/org.kde.passworddialog/contents/ui/Greeter.qml"
   local LOCK_SCREEN_FILE_PATH_ORG="/usr/share/kde4/apps/ksmserver/screenlocker/org.kde.passworddialog/contents/ui/Greeter.qml.org"
   local SWITCH_USER_FILE_PATH="/usr/share/kde4/services/plasma-runner-sessions.desktop"
   local SWITCH_USER_FILE_PATH_ORG="/usr/share/kde4/services/plasma-runner-sessions.desktop.org"
   local LOCK_SCREEN_FILE_PATH_PLASMA_5="/usr/share/plasma/look-and-feel/org.kde.breeze.desktop/contents/lockscreen/MainBlock.qml"
   local LOCK_SCREEN_FILE_PATH_PLASMA_5_ORG="/usr/share/plasma/look-and-feel/org.kde.breeze.desktop/contents/lockscreen/MainBlock.qml.org"
   local SWITCH_USER_FILE_PATH_PLASMA_5="/usr/share/plasma/look-and-feel/org.kde.breeze.desktop/contents/userswitcher/UserSwitcher.qml"
   local SWITCH_USER_FILE_PATH_PLASMA_5_ORG="/usr/share/plasma/look-and-feel/org.kde.breeze.desktop/contents/userswitcher/UserSwitcher.qml.org"
   local SWITCH_USER_IN_UNLOCK_SLED11SP4="/etc/dbus-1/system.d/ConsoleKit.conf"
   local SWITCH_USER_IN_UNLOCK_SLED11SP4_ORG="/etc/dbus-1/system.d/ConsoleKit.conf.org"

   if [ "$#" != "1" ]; then
      echo "customise_desktop_kde: specify operation"
      return
   fi

   mode="$1"

   # sled11sp4 kde hide switch user button in unlock pannel
   if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ] && [ "$OS_MAJOR" = "11" ]; then
      case "$mode" in
        "set")
           if [ -f $SWITCH_USER_IN_UNLOCK_SLED11SP4 ]; then
              mv -f $SWITCH_USER_IN_UNLOCK_SLED11SP4 $SWITCH_USER_IN_UNLOCK_SLED11SP4_ORG
              cp -fp $VIEWAGENT_PATH/resource/kde/ConsoleKit.conf $SWITCH_USER_IN_UNLOCK_SLED11SP4
           fi
           ;;
        "reset")
           if [ -f $SWITCH_USER_IN_UNLOCK_SLED11SP4_ORG ]; then
              mv -f $SWITCH_USER_IN_UNLOCK_SLED11SP4_ORG $SWITCH_USER_IN_UNLOCK_SLED11SP4
           fi
           ;;
        *)
           echo "customise_desktop_kde: specify set or reset."
           ;;
     esac
   fi

   case "$mode" in
   "set")
      # Disable Switch User in the lock screen
      if [ -f "$LOCK_SCREEN_FILE_PATH" ]; then
         # Backup system original file
         mv -f $LOCK_SCREEN_FILE_PATH $LOCK_SCREEN_FILE_PATH_ORG

         # Deploy new file
         cp -fp $VIEWAGENT_PATH/resources/kde/Greeter.qml $LOCK_SCREEN_FILE_PATH
      fi
      # Disable Switch User in the desktop
      if [ -f "$SWITCH_USER_FILE_PATH" ]; then
         # Backup system original file
         mv -f $SWITCH_USER_FILE_PATH $SWITCH_USER_FILE_PATH_ORG

      fi

      # Disable Switch User in lock screen for plasma 5
      if [ -f "$LOCK_SCREEN_FILE_PATH_PLASMA_5" ]; then
         # Backup system original file
         mv -f $LOCK_SCREEN_FILE_PATH $LOCK_SCREEN_FILE_PATH_PLASMA_5_ORG

         # Deploy new file
         cp -fp $VIEWAGENT_PATH/resources/kde/MainBlock.qml $LOCK_SCREEN_FILE_PATH_PLASMA_5
      fi

      # Disable New Session dialog for plasma 5
      if [ -f "$SWITCH_USER_FILE_PATH_PLASMA_5" ]; then
         # Backup system original file
         mv -f $SWITCH_USER_FILE_PATH_PLASMA_5 $SWITCH_USER_FILE_PATH_PLASMA_5_ORG
      fi
      ;;
   "reset")
      # Restore the system original file
      if [ -f "$LOCK_SCREEN_FILE_PATH_ORG" ]; then
         mv -f $LOCK_SCREEN_FILE_PATH_ORG $LOCK_SCREEN_FILE_PATH
      fi
      if [ -f "$SWITCH_USER_FILE_PATH_ORG" ]; then
         # Backup system original file
         mv -f $SWITCH_USER_FILE_PATH_ORG $SWITCH_USER_FILE_PATH

      fi

      if [ -f "$LOCK_SCREEN_FILE_PATH_PLASMA_5_ORG" ]; then
         mv -f $LOCK_SCREEN_FILE_PATH_PLASMA_5_ORG $LOCK_SCREEN_FILE_PATH_PLASMA_5
      fi

      if [ -f "$SWITCH_USER_FILE_PATH_PLASMA_5_ORG" ]; then
         mv -f $SWITCH_USER_FILE_PATH_PLASMA_5_ORG $SWITCH_USER_FILE_PATH_PLASMA_5
      fi
      ;;
   *)
      echo "customise_desktop_kde: specify set or reset"
      ;;
   esac
}


customise_desktop_NeoKylin() {
   local managed

   if [ "$#" != "1" ]; then
      echo "customise_desktop_NeoKylin: specify operation"
      return
   fi

   managed="$1"

   # Disable restart/shutdown/switchuser/suspend in the login UI of NeoKylin
   if [ "$managed" != "true" ]; then
      sed -i 's/RESTART_ENABLE\ *=\ *true/RESTART_ENABLE\ =\ false/g' /usr/share/gnome-shell/js/misc/config.js
      sed -i 's/SHUTDOWN_ENABLE\ *=\ *true/SHUTDOWN_ENABLE\ =\ false/g' /usr/share/gnome-shell/js/misc/config.js
   fi
   sed -i 's/OTHER_USER_ENABLE\ *=\ *\(true\|false\)/OTHER_USER_ENABLE\ =\ true/g' /usr/share/gnome-shell/js/misc/config.js
   sed -i 's/SUSPEND_ENABLE\ *=\ *true/SUSPEND_ENABLE\ =\ false/g' /usr/share/gnome-shell/js/misc/config.js

   # Disable restart/shutdown/switchuser/hibernate/suspend in the start menu of NeoKylin
   if [ "$managed" != "true" ]; then
      sed -i 's/shutdown\ *=\ *enable/shutdown=disable/g' /etc/mate-session-dialog.conf
      sed -i 's/reboot\ *=\ *enable/reboot=disable/g' /etc/mate-session-dialog.conf
   fi
   sed -i 's/switch_user\ *=\ *enable/switch_user=disable/g' /etc/mate-session-dialog.conf
   sed -i 's/hibernate\ *=\ *enable/hibernate=disable/g' /etc/mate-session-dialog.conf
   sed -i 's/suspend\ *=\ *enable/suspend=disable/g' /etc/mate-session-dialog.conf
}

customise_desktop_SLED() {
   local mode="$1"
   local managed="$2"
   local GDM_GREETER_LOGIN_WINDOW_GLAD="/usr/share/gdm/gdm-greeter-login-window.glade"
   local GDM_GREETER_LOGIN_WINDOW_GLAD_ORIG="$VIEWAGENT_PATH/resources/suse/gdm-greeter-login-window.glade.origin"
   local GNOME_SESSION_SHUTDOWN_DESKTOP="/usr/share/applications/gnome-session-shutdown.desktop"
   local GNOME_SESSION_SHUTDOWN_DESKTOP_ORIG="/usr/share/applications/gnome-session-shutdown.desktop.orig"
   local YAST2_X11_DESKTOP="/usr/share/applications/YaST2/x11.desktop"
   local YAST2_X11_DESKTOP_ORIG="/usr/share/applications/YaST2/x11.desktop.orig"
   local XDG_AUTOSTART_PATH="/etc/xdg/autostart"
   local VIEW_ETC="/etc/vmware"

   case "$mode" in
   "set")

      # fix the reset login window failed issue
      mv -f $GDM_GREETER_LOGIN_WINDOW_GLAD_ORIG $GDM_GREETER_LOGIN_WINDOW_GLAD

      # fix the reset start menu issue
      if [ -f $GNOME_SESSION_SHUTDOWN_DESKTOP_ORIG ]; then
         mv -f $GNOME_SESSION_SHUTDOWN_DESKTOP_ORIG $GNOME_SESSION_SHUTDOWN_DESKTOP
      fi

      # Disable restart/shutdown in the login UI for unmanaged agent
      if [ "$managed" != "true" ] &&
         [ -f "$GDM_GREETER_LOGIN_WINDOW_GLAD" ]; then
         # Backup system original file
         mv -f $GDM_GREETER_LOGIN_WINDOW_GLAD $VIEW_ETC/gdm-greeter-login-window.glade.orig

         # Deploy new file
         cp -fp $VIEWAGENT_PATH/resources/suse/gdm-greeter-login-window.glade $GDM_GREETER_LOGIN_WINDOW_GLAD
      fi

      # Disable shutdown/restart/hibernate/suspend for unmanaged agent in the  start menu of SLED
      if [ "$managed" != "true" ] &&
         [ -f "$GNOME_SESSION_SHUTDOWN_DESKTOP" ]; then
         mv -f $GNOME_SESSION_SHUTDOWN_DESKTOP $GNOME_SESSION_SHUTDOWN_DESKTOP_ORIG
      fi

      # Disable feature "Graphics Card and Monitor" in Control Center
      if [ -f "$YAST2_X11_DESKTOP" ]; then
         mv -f $YAST2_X11_DESKTOP $YAST2_X11_DESKTOP_ORIG
      fi

      if [ -f "$XDG_AUTOSTART_PATH/pulseaudio.desktop" ]; then
         cp -fp $XDG_AUTOSTART_PATH/pulseaudio.desktop $XDG_AUTOSTART_PATH/pulseaudio.desktop.orig
         sed -i "s/Exec=start-pulseaudio-x11/Exec=\/usr\/lib\/vmware\/viewagent\/resources\/suse\/start-pulseaudio-vmw/g" $XDG_AUTOSTART_PATH/pulseaudio.desktop
         chmod 755 $VIEWAGENT_PATH/resources/suse/start-pulseaudio-vmw
      fi
      ;;
   "reset")
      # Enable restart/shutdown in the login UI
      if [ "$managed" != "true" ] &&
         [ -f "$VIEW_ETC/gdm-greeter-login-window.glade.orig" ]; then
         mv -f $VIEW_ETC/gdm-greeter-login-window.glade.orig $GDM_GREETER_LOGIN_WINDOW_GLAD
      fi

      # Enable shutdown/restart/hibernate/suspend in the start menu of SLED
      if [ "$managed" != "true" ] &&
         [ -f "$GNOME_SESSION_SHUTDOWN_DESKTOP_ORIG" ]; then
         mv -f $GNOME_SESSION_SHUTDOWN_DESKTOP_ORIG $GNOME_SESSION_SHUTDOWN_DESKTOP
      fi

      # Enable feature "Graphics Card and Monitor" in Control Center
      if [ -f "$YAST2_X11_DESKTOP_ORIG" ]; then
         mv -f $YAST2_X11_DESKTOP_ORIG $YAST2_X11_DESKTOP
      fi

      if [ -f "$XDG_AUTOSTART_PATH/pulseaudio.desktop.orig" ]; then
         mv -f $XDG_AUTOSTART_PATH/pulseaudio.desktop.orig $XDG_AUTOSTART_PATH/pulseaudio.desktop
      fi
      ;;
   *)
      echo "customise_desktop_SLED: specify set or reset"
      ;;
   esac
}

customise_desktop_SLES() {
   local mode="$1"
   local SYSTEM_JS="/usr/share/gnome-shell/js/ui/status/system.js"
   local SYSTEM_JS_ORG="/usr/share/gnome-shell/js/ui/status/system.js.org"
   local GNOME_POWER_PANEL_DESKTOP="/usr/share/applications/gnome-power-panel.desktop"
   local GNOME_POWER_PANEL_DESKTOP_ORG="/usr/share/applications/gnome-power-panel.desktop.org"
   local ENABLE_SWITCH_USER="let visible = allowSwitch && multiUser"
   local DISABLE_SWITCH_USER="let visible = false"

   case "$mode" in
   "set")
      # Disable Switch User in the desktop
      if [ -f "$SYSTEM_JS" ]; then
         # Backup system original file
         mv -f $SYSTEM_JS $SYSTEM_JS_ORG

         # Deploy new file
         sed "s/$ENABLE_SWITCH_USER/$DISABLE_SWITCH_USER/" $SYSTEM_JS_ORG >> $SYSTEM_JS
      fi
      # Disable Automatic suspend button in the control center
      if [ -f "$GNOME_POWER_PANEL_DESKTOP" ]; then
         mv -f $GNOME_POWER_PANEL_DESKTOP $GNOME_POWER_PANEL_DESKTOP_ORG
      fi
      ;;
   "reset")
      # Enable Switch User in the desktop
      if [ -f "$SYSTEM_JS_ORG" ]; then
         mv -f $SYSTEM_JS_ORG $SYSTEM_JS
      fi
      # Enable Automatic suspend button in the control center
      if [ -f "$GNOME_POWER_PANEL_DESKTOP_ORG" ]; then
         mv -f $GNOME_POWER_PANEL_DESKTOP_ORG $GNOME_POWER_PANEL_DESKTOP
      fi
      ;;
   *)
      echo "customise_desktop_SLES: specify set or reset"
      ;;
   esac
}


#
# Desktop session command functions
#

parse_X_args() {
   local ret=1

   shift

   while [ "$#" -gt 0 ]
   do
      if [ "`expr substr $1 1 1`" = ":" ]; then
         export DISPLAY="$1"
      elif [ "$1" = "-auth" ]; then
         shift
         export XAUTHORITY="$1"
      fi
      shift
   done

   #
   # Workaround for an issue in the KUbuntu16.04.
   # On the KUbuntu16.04, we can't get the DISPLAY from the Xorg process information:
   # /usr/lib/xorg/Xorg -nolisten tcp -auth /var/run/sddm/{ca341de6-b935-42d3-9b63-fb48bb9ff7aa} -background none -noreset -displayfd 18 vt7
   # As the linux agent only support the single session, we can get the DISPLAY from the domain socket in the directory "/tmp/.X11-unix/".
   # https://unix.stackexchange.com/questions/196677/what-is-tmp-x11-unix
   # https://unix.stackexchange.com/questions/17255/is-there-a-command-to-list-all-open-displays-on-a-machine
   # https://stackoverflow.com/questions/11367354/obtaining-list-of-all-xorg-displays
   #
   if [ -z "$DISPLAY" ]; then
      xUnixSockets=$(ls /tmp/.X11-unix/ | egrep ^X[0-9]+)
      for xSocket in $xUnixSockets
      do
         xSocket="${xSocket#X}"
         if [ -n "$xSocket" ]; then
            export DISPLAY=":$xSocket"
            break
         fi
      done
   fi

   [ -n "$DISPLAY" ] && [ -n "$XAUTHORITY" ] && ret=0

   return $ret
}


extract_X_args() {
   local xpid
   local xcmd
   #
   # For multi session the following two environment variables shall need to
   # vary to suit the session required.   Single session per VM for now...
   #
   # Keyboard mapping should be set here for VMwareBlastServer?
   #
   xpid="`pidof X`"
   [ "$?" = "0" ] || xpid="`pidof Xorg`"
   [ "$?" = "0" ] && xcmd="`ps -p $xpid -o cmd --no-headers`" && \
                     parse_X_args $xcmd
}


configure_xinput() {
   local val

   if [ "$#" -gt 0 ]; then
      case $1 in
      disable)
         val=0;;
      enable)
         val=1;;
      *)
         echo "Failed to configure xinput device: unknown the params $1"
         return
         ;;
      esac
   fi

   #
   # Disable the physics input driver via xinput
   #
   while read devname
   do
      xinput set-prop "$devname" 'Device Enabled' $val >/dev/null 2>&1
   done << !!!
`xinput --list --name-only| grep -v "^Virtual core"`
!!!
}


configure_xorg() {
   local res="1280x1024"

   [ "$#" -gt 0 ] && res=$1

   #
   # Switch to the requested screen resolution, where available.
   # TODO: Add temporary mode to screen to handle arbitrary resolutions.
   #
   xrandr --output Virtual1 --mode $res >/dev/null 2>&1
}


#
# Do lock/unlock the desktop
#
configure_session_lock() {
   local params
   local dbus_params
   local UserName
   local desktop_type_file
   local desktop_type

   if [ "$#" -gt 0 ]; then
      case $1 in
      unlock)
         params="-d"
         dbus_params="false"
         ;;
      lock)
         params="-l"
         dbus_params="true"
         ;;
      *)
         echo "Failed to lock/unlock the session: unknown the params $1"
         return
         ;;
      esac

      if [ -n "$2" ]; then
         UserName="$2"
      else
         echo "Failed to lock/unlock the session: missing parameter username"
         return
      fi
   fi

   desktop_type_file="/var/vmware/view.${UserName}/desktopType"
   if [ -f $desktop_type_file ]; then
      desktop_type="`cat ${desktop_type_file}`"
   fi

   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS} |\
   ${DISTRO_ID_RHEL_CLIENT} |\
   ${DISTRO_ID_RHEL_SERVER} |\
   ${DISTRO_ID_RHEL_WORKSTATION})
      if [ ${OS_MAJOR} = "6" ] && [ "$desktop_type" = "kde" ]; then
         if [ "$dbus_params" = "true" ]; then
            su "$UserName" -c "dbus-send --type=method_call --dest=org.freedesktop.ScreenSaver /ScreenSaver org.freedesktop.ScreenSaver.Lock" >/dev/null 2>&1 && return
         fi
         if [ "$dbus_params" = "false" ]; then
            su "$UserName" -c "dbus-send --type=method_call --dest=org.freedesktop.ScreenSaver /ScreenSaver org.freedesktop.ScreenSaver.SetActive boolean:false" >/dev/null 2>&1 && return
         fi
      fi
      if [ ${OS_MAJOR} -ge "7" ]; then
         su "$UserName" -c "dbus-send --session --dest=org.gnome.ScreenSaver --type=method_call /org/gnome/ScreenSaver org.gnome.ScreenSaver.SetActive boolean:${dbus_params}" >/dev/null 2>&1 && return
      fi
      ;;
   ${DISTRO_ID_UBUNTU})
      if [ "${OS_MAJOR}" = "14" ] || [ "${OS_MAJOR}" = "16" ]; then
         userSessions=`loginctl list-sessions | grep -F "$UserName" | grep -i "seat" | awk '{ print $1}' | xargs`
         echo "current user active session id=$userSessions"
         if [ "$1" = "unlock" ]; then
	    loginctl unlock-session $userSessions > /dev/null 2>&1 && return
	 elif [ "$1" = "lock" ]; then
	    loginctl lock-session $userSessions > /dev/null 2>&1 && return
	 fi
      fi
      ;;
   *)
      ;;
   esac

   which mate-screensaver-command >/dev/null 2>&1
   [ "$?" = "0" ] && su "$UserName" -c "mate-screensaver-command ${params}" >/dev/null 2>&1 && return
   which gnome-screensaver-command >/dev/null 2>&1
   [ "$?" = "0" ] && su "$UserName" -c "gnome-screensaver-command ${params}" >/dev/null 2>&1 && return

   echo "Failed to $1 desktop session"
}


disable_xorg_VTSwitch()
{
   local xorg_conf="/etc/X11/xorg.conf"

   if [ ! -f $xorg_conf ]; then
      echo "disable_xorg_VTSwith: $xorg_conf doesn't exist"
      return
   fi

   nVT=`grep \"DontVTSwitch\" $xorg_conf |wc -l`
   if [ $nVT -ge 1 ]; then
      sed '/\"DontVTSwitch\"/ c\     Option    \"DontVTSwitch\"    \"true\"' $xorg_conf > $xorg_conf.tmp
      mv -f $xorg_conf.tmp $xorg_conf
      return
   fi

   nSF=`grep \"ServerFlags\" $xorg_conf | wc -l`
   if [ $nSF -ge 1 ]; then
      sed '/\"ServerFlags\"/ a\     Option    \"DontVTSwitch\"    \"true\"' $xorg_conf > $xorg_conf.tmp
      mv -f $xorg_conf.tmp $xorg_conf
      return
   fi

   echo "Section    \"ServerFlags\"" >> $xorg_conf
   echo "    Option    \"DontVTSwitch\"    \"true\"" >> $xorg_conf
   echo "EndSection"  >> $xorg_conf
}


config_xorg_vDGA()
{
   local xorg_conf="$XORG_CONF_DIR/xorg.conf"
   local edid_path="/usr/lib/vmware/viewagent/resources/X11/edid.bin"
   local delflag=0
   local ifs=$IFS
   local line
   local lastline
   local mode
   local pci_entry
   local pci_bus
   local pci_device
   local pci_function

   if [ ! -f $xorg_conf ]; then
      echo "config_xorg_vDGA: $xorg_conf doesn't exist"
      return
   fi

   mode="$1"

   nNVDIA=`lspci -v |grep "Kernel driver in use: nvidia" |wc -l`
   nEDID=`grep "/etc/X11/edid.bin" $xorg_conf |wc -l`

   #
   # If the content of xorg.conf is accordant with lspci,
   # We don't need config the xorg.conf again.
   #
   if [ $nNVDIA -eq $nEDID ]; then
      #
      # If install viewagent in vDGA mode, we force to update
      # the xorg.conf file.
      #
      if [ "$mode" != "install" ] || [ $nNVDIA -eq 0 ]; then
         return
      fi
   fi

   #
   # Remove the old "Device0" configure
   #
   IFS=
   while read line
   do
      if echo $line |grep -q "\"Device0\"" && echo $line|grep -q "Identifier"; then
         delflag=1
         continue
      elif echo $line |grep -q "EndSection"; then
         if [ $delflag -eq 1 ]; then
            lastline=
            delflag=0
            continue
         fi
      fi
      if [ $delflag -eq 0 ] && [ -n "$lastline" ]; then
         echo $lastline >> $xorg_conf.tmp
      fi
      lastline="$line"
   done < $xorg_conf
   if [ $delflag -eq 0 ] && [ -n "$lastline" ]; then
         echo $lastline >> $xorg_conf.tmp
   fi
   IFS=$ifs
   mv $xorg_conf.tmp $xorg_conf

   #
   # If the VM has vDGA driver, add the vDGA configure to xorg.conf
   #
   if [ $nNVDIA -ge 1 ]; then
      if [ ! -e "$XORG_CONF_DIR/edid.bin" ]; then
         cp -fp /usr/lib/vmware/viewagent/resources/X11/edid.bin $XORG_CONF_DIR/edid.bin
      else
         echo "configure_xorg_vDGA: $XORG_CONF_DIR/edid.bin have exist."
      fi
      pci_entry=`lspci |grep "NVIDIA" |cut -d" " -f1`
      pci_bus=`echo $pci_entry |awk -F'[:.]' '{print $1}'`
      pci_device=`echo $pci_entry |awk -F'[:.]' '{print $2}'`
      pci_function=`echo $pci_entry |awk -F'[:.]' '{print $3}'`
      pci_entry=`printf "%d:%d:%d" 0x${pci_bus} 0x${pci_device} 0x${pci_function}`
      echo "Section    \"Device\"" >> $xorg_conf
      echo "    Identifier \"Device0\"" >> $xorg_conf
      echo "    Driver     \"nvidia\"" >> $xorg_conf
      echo "    VendorName \"NVIDIA Corporation\"" >> $xorg_conf
      echo "    BusID      \"PCI:$pci_entry\"" >> $xorg_conf
      echo "    Option     \"IgnoreDisplayDevices\" \"CRT-0\"" >> $xorg_conf
      echo "    Option     \"NoFlip\" \"TRUE\"" >> $xorg_conf
      echo "    Option     \"ConnectedMonitor\" \"DFP-0, DFP-1, DFP-2, DFP-3\"" >> $xorg_conf
      echo "    Option     \"CustomEDID\" \"DFP-0:/etc/X11/edid.bin; DFP-1:/etc/X11/edid.bin; DFP-2:/etc/X11/edid.bin; DFP-3:/etc/X11/edid.bin\"" >> $xorg_conf
      echo "    Option     \"ModeValidation\" \"NoVesaModes,NoPredefinedModes,NoXserverModes,NoMaxSizeCheck,NoEdidMaxPClkCheck,NoHorizSyncCheck,NoVertRefreshCheck,AllowNonEdidModes\"" >> $xorg_conf
      echo "EndSection"  >> $xorg_conf
   fi
}


configure_xorg_conf()
{
   umask 022
   local xorg_conf="$XORG_CONF_DIR/xorg.conf"
   local mode
   local vDGA_flag=0

   if [ "$#" != "1" ]; then
      echo "configure_xorg_conf: specify operation"
      return
   fi

   lspci -v |grep "Kernel driver in use: nvidia"
   if [ "$?" = "0" ]; then
      vDGA_flag=1
   fi

   mode="$1"

   case "$mode" in
   "reset")
      #
      # If linux agent is in vDGA mode, we don't revert
      # the xorg.conf file.
      #
      if [ $vDGA_flag -eq 1 ]; then
         return
      fi

      #
      # We backup current xorg.conf file.
      #
      if [ -f $xorg_conf ]; then
         mv $xorg_conf $XORG_CONF_DIR/xorg.conf.vmware_backup
      fi

      if [ -s $VIEWAGENT_PATH/xorg.conf.orig ]; then
         mv -f $VIEWAGENT_PATH/xorg.conf.orig $xorg_conf
      fi

      if [ -f $XORG_CONF_DIR/edid.bin ]; then
         rm -f $XORG_CONF_DIR/edid.bin
      fi
      ;;
   "set" | "install")
      if [ ! -d $XORG_CONF_DIR ]; then
         mkdir -p $XORG_CONF_DIR
      fi

      if [ ! -f $xorg_conf ]; then
         touch $xorg_conf
      fi

      if [ ! -f $VIEWAGENT_PATH/xorg.conf.orig ]; then
         if [ $vDGA_flag -eq 0 ]; then
            cp -fp $xorg_conf $VIEWAGENT_PATH/xorg.conf.orig
         elif [ -f $XORG_CONF_DIR/xorg.conf.nvidia-xconfig-original ]; then
            #
            # If linux agent is in vDGA mode, we save the original
            # xorg.conf of NVIDIA
            #
            cp -f $XORG_CONF_DIR/xorg.conf.nvidia-xconfig-original $VIEWAGENT_PATH/xorg.conf.orig
         fi
      fi

      disable_xorg_VTSwitch
      config_xorg_vDGA $mode
      ;;
   esac
}


#
# If VM is configured as 2D/vSGA, reminder user to check document for
# vRAM Settings for Linux Guest Operating Systems
#
show_vRAM_Settings()
{
   nNVDIA=`lspci -v |grep "Kernel driver in use: nvidia"`
   if [ "$?" != "0" ]; then
      echo "Your VM is configured as 2D/vSGA, please check guide for \"Recommended Video Memory (vRAM) Settings\"."
      #
      # we should disable XVideo extension, otherwise it will will show black to Horizon client when playing video.
      #
      if [ -f /etc/X11/xorg.conf ]; then
         echo "Section    \"Extensions\"" >> /etc/X11/xorg.conf
         echo "     Option    \"XVideo\"    \"Disable\"" >> /etc/X11/xorg.conf
         echo "EndSection" >> /etc/X11/xorg.conf
      fi
   fi
}


check_greeter_status_file()
{
   if [ "$#" != "1" ]; then
      echo "check_greeter_status_file: specify X display manager"
      return
   fi

   dm="$1"
   GREETER_STATUS_FILE="/var/vmware/greeter/greeter_status"
   if [ ! -f "$GREETER_STATUS_FILE" ]; then
      touch $GREETER_STATUS_FILE
      case "$dm" in
         "gdm")
            chown gdm:gdm $GREETER_STATUS_FILE
            ;;
         "lightdm")
            chown lightdm:lightdm $GREETER_STATUS_FILE
            ;;
      esac
      chmod 600 $GREETER_STATUS_FILE
      which getenforce >/dev/null 2>&1
      if [ "$?" = "0" ]; then
         status=`getenforce`
         if [ "$status" != "Disabled" ]; then
            chcon -u system_u -r object_r -t xdm_log_t -l s0 $GREETER_STATUS_FILE
         fi
      fi
   fi
}


check_environment_whitelisted()
{
   case "$1" in
      "AUDIODRIVER" |\
      "COMPIZ_CONFIG_PROFILE" |\
      "DESKTOP_AUTOSTART_ID" |\
      "DBUS_SESSION_BUS_ADDRESS" |\
      "DISPLAY" |\
      "GDMSESSION" |\
      "GDM_LANG" |\
      "GNOME_DESKTOP_SESSION_ID" |\
      "GNOME_SHELL_SESSION_MODE" |\
      "GTK_MODULES" |\
      "HOSTNAME" |\
      "LANG" |\
      "PWD" |\
      "SHELL" |\
      "SESSION_MANAGER" |\
      "SESSIONTYPE" |\
      "SSH_AGENT_PID" |\
      "UPSTART_SESSION" |\
      "USER" |\
      "USERNAME" |\
      "VMW_SESSION_ID" |\
      "VMW_MONITOR_ID" |\
      "WINDOWMANAGER" |\
      "XAUTHORITY" |\
      "XDG_VTNR" |\
      "XDG_SESSION_ID" |\
      "XDG_SEAT" |\
      "XDG_SEAT_PATH" |\
      "XDG_SESSION_PATH" |\
      "XDG_RUNTIME_DIR" |\
      "XDG_MENU_PREFIX" |\
      "XDG_CURRENT_DESKTOP" |\
      "XDG_GREETER_DATA_DIR" |\
      "XDG_CONFIG_DIRS" |\
      "XDG_DATA_DIRS" |\
      "XDG_SESSION_DESKTOP" |\
      "XSESSION_IS_UP" |\
      "_")
         return 0
         ;;
      *)
         return 1
         ;;
   esac
}


directory_validity_check()
{
   dir=$1
   owner=$2
   group=$3
   mode=$4

   if [ -d "$dir" ]; then
      dirowner=`ls -ld $dir | awk '{print $3}'`
      if [ "$dirowner" != "$owner" ]; then
         echo "$dir exists, but the file owner doesn't match $owner"
         exit 18
      fi

      if [ "$mode" != "0" ]; then
         dirmode=`stat -c %a $dir`
         if [ "$dirmode" != "$mode" ]; then
            echo "$dir exists, but the file mode doesn't match $mode"
            exit 18
         fi
      fi
   else
      mkdir -m $mode $dir
      if [ "$?" != "0" ]; then
         echo "fails to create $dir"
         exit 18
      fi
      chown $owner:$group $dir
   fi
}


verify_udev_environment() {
   local meet_libudev=0
   local path_libudev

   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS} |\
   ${DISTRO_ID_RHEL_CLIENT} |\
   ${DISTRO_ID_RHEL_SERVER} |\
   ${DISTRO_ID_RHEL_WORKSTATION})
      if [ ${OS_MAJOR} -ge "7" ]; then
         meet_libudev=1
      fi
      ;;

   ${DISTRO_ID_SUSE})
      if [ ${OS_MAJOR} -ge "12" ]; then
         meet_libudev=1
      fi
      ;;

   ${DISTRO_ID_UBUNTU})
      if [ ${OS_MAJOR} -ge "14" ]; then
         meet_libudev=1
      fi
      ;;

   *)
      ;;
   esac

   if [ "${meet_libudev}" = "1" ]; then
      #
      # We only support 64-bit system
      #
      ldconfig -p | grep libudev.so | grep "x86-64" >/dev/null 2>&1
      if [ "$?" = "0" ]; then
         path_libudev=`ldconfig -p | grep libudev.so | grep "x86-64" | awk 'NR==1{print $4}'`
         if [ -f ${path_libudev%/*}/libudev.so.0 ]; then
            meet_libudev=0
         else
            [ -f ${path_libudev} ] && \
            ln -sf ${path_libudev} ${path_libudev%/*}/libudev.so.0
            if [ "$?" = "0" ]; then
               meet_libudev=0
            fi
         fi
      fi
   fi

   return ${meet_libudev}
}
