#!/bin/sh

###################################################
# Copyright 2015 VMware, Inc.  All rights reserved.
###################################################

#
# The following settings replicate the difference before and after when
# disabling animations and fading windows effects via
# compizconfig-settings-manager as identified using "dconf dump /" on
# Ubuntu 14.04.
#
# Note, when adding further settings some of the difference is accounted for by
# application defaults appearing in dconf following ccsm execution, not as
# a direct function of disabling effects.
#
# The purpose of this script is to perform the equivalent of ccsm disabling
# animations and fading windows effects without requiring GUI interaction.
#

disable_effects() {
   if [ -f "$DC_BACKUP" ]; then
      return
   fi

   dconf dump / > $DC_BACKUP

   echo "Notice: original dconf dumped to $DC_BACKUP,"

   gsettings set org.compiz.core:/org/compiz/profiles/Default/plugins/core/ active-plugins "['core', 'composite', 'opengl', 'scale', 'vpswitch', 'place', 'copytex', 'snap', 'commands', 'compiztoolbox', 'wall', 'mousepoll', 'regex', 'imgpng', 'resize', 'grid', 'workarounds', 'move', 'unitymtgrabhandles', 'session', 'expo', 'ezoom', 'unityshell']"
}


usage() {
   echo "Usage: compiz-effects.sh disable"
}


#
# main
#

if [ "$#" != "1" ]; then
   echo "compiz-effects.sh disable"
   exit
fi

mode="$1"

DC_BACKUP=~/dconf.vmw.backup

case "$mode" in
"disable")
   disable_effects
   ;;
*)
   usage
   ;;
esac
